/**************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto , (c)2004 by 421                 *
*   ikemo@wakaba.jp                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#ifndef KITADATMG_H
#define KITADATMG_H

#include <qstringlist.h>
#include <kurl.h>
#include <qmap.h>

class QSemaphore;

namespace Kita
{

#define DMANAGER_MAXQUEUE 15

    enum{
        ABONECHK_BY_ID,
        ABONECHK_BY_NAME,
        ABONECHK_BY_WORD
    };

/* Database for anchor information */ 
/* For example, 5-th res has anchors >>2-3 and >>4, then,

   m_anclistmap[5].at(0).from = 2,
   m_anclistmap[5].at(0).to = 3,
   m_anclistmap[5].at(1).from = 4,
   m_anclistmap[5].at(1).from = 4.   

*/
struct ANCNUM{
    int from;
    int to;
};
typedef QValueList<ANCNUM> AncList;
typedef QMap<int,AncList> AncListMap;    

    /*-----------------------*/

    class DatInfo
    {
        int m_maxNum;
        KURL m_url;
        QString m_subject;
        QStringList m_linedat;
        QStringList m_idlist;
    AncListMap m_anclistmap;

    QString m_spacestr;
    QString m_framestr1; /* '|' */
    QString m_framestr2; /* '|-' */
    QString m_framestr3; /* 'L' */
    
    public:
        DatInfo();
        DatInfo( const KURL& url );
        const QString& getDat( int num );
        const QString& getId( int num );
        const QString& getSubject() { return m_subject;}
        KURL& url() { return m_url;};
        int getMaxResNumber() { return m_maxNum;}
	QString getHtml(int num);
	bool checkRes(const  int target, const int num );
	QString getTreeByRes(const int rootnum, int& num, QString prestr );
    };

    /*-----------------------*/
    typedef QValueList<DatInfo*> DatInfoList;

    class DatManager
    {
        static DatInfoList m_datInfo;
        static QSemaphore m_semap;

    public:
        static void deleteDat( const KURL& url );
        static const QString& getDat( const KURL& url, int num );
        static const QString& getSubject( const KURL& url );
        static QString getHtml( const KURL& url, int startnum, int endnum );
        static int getMaxResNumber( const KURL& url );
	static int getNumByID(const KURL& url, const QString& strid );
	static QString getHtmlByID(const KURL& url, const QString& strid, int &num );
	static QString getTreeByRes(const KURL& url, const int resNum, int &num );
	static bool checkID(const KURL& url, const QString& strid, int num );
	static bool checkWord(const KURL& url,QStringList& stlist, int num, bool checkOR);
	static bool checkRes(const KURL& url, const int target,const int num );

        static bool checkAbone( const QString& str, int mode );
    static bool parseResAnchor(const QChar *cdat,const unsigned int length,QString& linkstr,int* refNum,unsigned int& pos);
    private:
        static DatInfo* getDatInfo( const KURL& url );
    };


}

#endif
